import unittest

from sage.all import *

from eta_products_helper import span_with_eta_products, find_eta_products

class TestEtaProductHelper ( unittest.TestCase ) :
    
    def setUp(self) :
        pass

    def test__span_with_eta_products(self) :
        for (level, weight, prec, helper_level) in [ (5, 8, 50, 1),
                                                     (49, 4, 50, 1),
                                                     (16, 2, 50, 4) ] :
            mf = ModularForms(level, weight)
            self.assertTrue(all(
                    mf(f.eval_projection(prec)) is not None
                    for f in span_with_eta_products(level, weight, helper_level)[1] ))

    def test__find_eta_products(self) :
        self.assertEqual( find_eta_products(23,2),
                          [(0, 4), (1, 3), (2, 2), (3, 1), (4, 0)] )
        self.assertEqual( find_eta_products(12,Integer(1)/2),
                          [(2, -1, 0, 0, 0, 0), (0, 0, 2, 0, -1, 0), (-2, 5, 1, -2, -2, 1),
                           (0, 2, 0, -1, -1, 1), (0, 0, 1, 0, -1, 1), (0, 0, 0, 0, -1, 2),
                           (-1, 2, 1, 0, -1, 0), (1, -1, 0, 1, 0, 0), (0, -1, 0, 2, 0, 0),
                           (0, 1, 0, -1, -1, 2), (1, 0, 0, 0, 0, 0), (-1, 1, 2, 0, -1, 0),
                           (0, 0, 0, 1, 0, 0), (-1, 2, 1, -1, -1, 1), (0, 1, 0, 0, 0, 0),
                           (0, 2, 0, -1, 0, 0), (2, -1, -1, 0, 1, 0), (0, 0, 1, 0, 0, 0),
                           (0, 0, 0, 0, 0, 1), (-1, 2, 0, 0, 0, 0), (0, -1, 0, 2, 1, -1),
                            (-1, 3, 0, -1, 0, 0), (-2, 5, 0, -2, 0, 0), (0, 0, 0, 0, 1, 0),
                           (1, -1, -1, 1, 2, -1), (1, -1, -1, 0, 2, 0), (0, -1, 0, 1, 2, -1),
                           (0, 0, 0, 0, 2, -1), (0, 0, -1, 0, 2, 0), (0, 0, -1, 0, 3, -1),
                           (1, -2, -2, 1, 5, -2), (0, 0, -2, 0, 5, -2)] )
        self.assertEqual( find_eta_products(30,1),
                          [(-2, 4, 1, 1, -2, -2, -2, 4), (6, -3, -2, 0, 1, 0, 0, 0), (-2, 3, 2, 2, -2, -2, -2, 3),
                           (-2, 2, 3, 3, -2, -2, -2, 2), (-2, 1, 4, 4, -2, -2, -2, 1), (0, 0, 0, 6, 0, -3, -2, 1),
                           (0, 0, 0, 1, 0, -2, -3, 6), (1, -2, -3, 0, 6, 0, 0, 0), (4, -2, -2, 0, 2, 0, 0, 0),
                           (2, -1, -2, 0, 3, 0, 0, 0), (0, 0, 3, 0, -1, 0, -1, 1), (2, -1, -1, 2, 1, -1, -1, 1),
                           (0, 0, -2, 0, 4, 0, 0, 0), (3, 0, -1, 0, 0, 0, 0, 0), (0, 0, -1, 2, 2, -1, -1, 1),
                           (0, 0, 0, 4, 0, -2, -2, 2), (1, 1, -1, 0, 1, 0, 0, 0), (-1, 2, -1, 0, 2, 0, 0, 0),
                           (-1, 2, 0, 2, 0, -1, -1, 1), (0, 3, 0, 0, -1, 0, 0, 0), (2, -1, -1, 0, 1, 0, -1, 2),
                           (3, -1, 0, -1, 0, 1, 0, 0), (-2, 4, 0, 0, 0, 0, 0, 0), (0, 0, -1, 0, 2, 0, -1, 2),
                           (0, 0, 0, 2, 0, -1, -2, 3), (-3, 6, 1, 0, -2, 0, 0, 0), (-1, 2, 0, 0, 0, 0, -1, 2),
                           (-1, 1, 1, 1, 0, 0, -1, 1), (-1, 0, 2, 2, 0, 0, -1, 0), (0, 0, 0, 0, 0, 0, -2, 4),
                            (0, 0, 0, 3, 0, 0, -1, 0), (2, -1, -1, -1, 1, 2, 0, 0), (0, 0, -1, -1, 2, 2, 0, 0),
                           (0, -1, 0, 0, 2, 2, 0, -1), (0, 0, 0, 1, 0, 1, -1, 1), (-1, 2, 0, -1, 0, 2, 0, 0),
                           (0, 0, 0, -1, 0, 2, -1, 2), (1, -2, -2, -2, 4, 4, 1, -2), (0, 0, 0, 0, 0, 3, 0, -1),
                           (0, 0, 0, -2, 0, 4, 0, 0), (0, 0, 0, -3, 0, 6, 1, -2), (-2, 3, 3, 1, -2, -1, -1, 1),
                           (3, -2, -2, 0, 3, 0, 0, 0), (1, -1, -2, 0, 4, 0, 0, 0), (1, -1, -1, 2, 2, -1, -1, 1),
                           (-1, 1, 3, 1, -2, -1, -2, 3), (2, 0, -1, 0, 1, 0, 0, 0), (0, 1, -1, 0, 2, 0, 0, 0),
                           (0, 1, 0, 2, 0, -1, -1, 1), (4, -2, 0, -1, 0, 1, 0, 0), (-1, 3, 0, 0, 0, 0, 0, 0),
                           (1, -1, -1, 0, 2, 0, -1, 2), (0, 1, 0, 0, 0, 0, -1, 2), (0, 0, 1, 1, 0, 0, -1, 1),
                           (1, -1, -1, -1, 2, 2, 0, 0), (0, 1, 0, -1, 0, 2, 0, 0), (1, -1, 0, -2, 0, 4, 0, 0), 
                           (2, -2, -2, 0, 4, 0, 0, 0), (3, -1, -1, 0, 1, 0, 0, 0), (1, 0, -1, 0, 2, 0, 0, 0),
                           (1, 0, 0, 2, 0, -1, -1, 1), (0, 2, 0, 0, 0, 0, 0, 0), (1, 0, 0, 0, 0, 0, -1, 2),
                           (1, 0, 0, -1, 0, 2, 0, 0), (4, -2, -1, 0, 1, 0, 0, 0), (2, -1, -1, 0, 2, 0, 0, 0),
                           (0, 0, 4, 0, -2, 0, -1, 1), (2, -1, 0, 2, 0, -1, -1, 1), (0, 0, -1, 0, 3, 0, 0, 0),
                           (0, 0, 0, 2, 1, -1, -1, 1), (1, 1, 0, 0, 0, 0, 0, 0), (-1, 2, 0, 0, 1, 0, 0, 0),
                           (-1, 2, 1, 2, -1, -1, -1, 1), (2, -1, 0, 0, 0, 0, -1, 2), (-2, 4, 1, 0, -1, 0, 0, 0),
                           (0, 0, 0, 0, 1, 0, -1, 2), (-1, 2, 1, 0, -1, 0, -1, 2), (-1, 1, 2, 1, -1, 0, -1, 1),
                           (0, 0, 1, 0, -1, 0, -2, 4), (2, -1, 0, -1, 0, 2, 0, 0), (0, 0, 0, -1, 1, 2, 0, 0),
                           (-1, 2, 1, -1, -1, 2, 0, 0), (3, -2, -1, 0, 2, 0, 0, 0), (1, -1, -1, 0, 3, 0, 0, 0),
                           (2, 0, 0, 0, 0, 0, 0, 0), (0, 1, 0, 0, 1, 0, 0, 0), (-1, 3, 1, 0, -1, 0, 0, 0),
                           (0, 1, 1, 0, -1, 0, -1, 2), (0, 0, 2, 1, -1, 0, -1, 1), (1, -1, 0, -1, 1, 2, 0, 0),
                           (2, -1, -1, 1, 1, -1, -1, 2), (3, -1, 0, 0, 0, 0, 0, 0), (-2, 4, 0, 1, 0, -1, 0, 0),
                           (0, 0, -1, 1, 2, -1, -1, 2), (0, 0, 0, 3, 0, -2, -2, 3), (1, 0, 0, 0, 1, 0, 0, 0),
                           (-1, 2, 0, 1, 0, -1, -1, 2), (-1, 1, 1, 2, 0, -1, -1, 1), (0, 2, 1, 0, -1, 0, 0, 0),
                           (0, 0, 0, 1, 0, -1, -2, 4), (2, -1, -1, 0, 1, 1, 0, 0), (0, 0, -1, 0, 2, 1, 0, 0),
                           (0, 0, 0, 2, 0, 0, -1, 1), (-1, 2, 0, 0, 0, 1, 0, 0), (0, 0, 0, 0, 0, 1, -1, 2),
                           (0, 0, 0, -1, 0, 3, 0, 0), (4, -2, 0, 0, 0, 0, 0, 0), (-1, 3, 0, 1, 0, -1, 0, 0),
                           (1, -1, -1, 1, 2, -1, -1, 2), (2, -1, 0, 0, 1, 0, 0, 0), (0, 0, 0, 0, 2, 0, 0, 0),
                           (0, 1, 0, 1, 0, -1, -1, 2), (0, 0, 1, 2, 0, -1, -1, 1), (1, 1, 1, 0, -1, 0, 0, 0),
                           (-1, 2, 1, 0, 0, 0, 0, 0), (-2, 4, 2, 0, -2, 0, 0, 0), (0, 0, 1, 0, 0, 0, -1, 2),
                           (1, -1, -1, 0, 2, 1, 0, 0), (0, 1, 0, 0, 0, 1, 0, 0), (0, 0, 1, -1, 0, 2, 0, 0),
                           (1, -1, 0, -1, 0, 3, 0, 0), (-2, 3, 1, 3, -1, -2, -1, 1), (1, -1, 0, 0, 2, 0, 0, 0),
                           (1, 0, 0, 1, 0, -1, -1, 2), (-1, 1, 1, 3, -1, -2, -2, 3), (0, 1, 1, 0, 0, 0, 0, 0),
                           (1, 0, 0, 0, 0, 1, 0, 0), (2, -1, 0, 1, 0, -1, -1, 2), (0, -1, 0, 0, 3, 0, 0, 0),
                           (0, 0, 0, 1, 1, -1, -1, 2), (1, 0, 1, 0, 0, 0, 0, 0), (-1, 1, 1, 0, 1, 0, 0, 0),
                           (-1, 2, 1, 1, -1, -1, -1, 2), (-1, 1, 2, 2, -1, -1, -1, 1), (-2, 3, 2, 0, -1, 0, 0, 0),
                           (-1, 1, 2, 0, -1, 0, -1, 2), (2, -1, 0, 0, 0, 1, 0, 0), (0, 0, 0, 0, 1, 1, 0, 0),
                           (-1, 2, 1, 0, -1, 1, 0, 0), (-1, 1, 2, -1, -1, 2, 0, 0), (2, -1, 1, 0, 0, 0, 0, 0),
                           (0, 0, 1, 0, 1, 0, 0, 0), (0, 1, 1, 1, -1, -1, -1, 2), (0, 0, 2, 2, -1, -1, -1, 1),
                           (-1, 2, 2, 0, -1, 0, 0, 0), (0, 0, 2, 0, -1, 0, -1, 2), (1, -1, 0, 0, 1, 1, 0, 0),
                           (0, 0, 2, -1, -1, 2, 0, 0), (0, 0, 0, 2, 0, -2, -2, 4), (1, -1, 1, 0, 1, 0, 0, 0),
                           (0, 1, 2, 0, -1, 0, 0, 0), (2, -1, -1, 1, 1, 0, 0, 0), (0, 0, -1, 1, 2, 0, 0, 0),
                           (0, 0, 0, 3, 0, -1, -1, 1), (-1, 2, 0, 1, 0, 0, 0, 0), (0, 0, 0, 1, 0, 0, -1, 2),
                           (0, 0, 0, 0, 0, 2, 0, 0), (0, 0, 1, 1, 0, -1, -1, 2), (1, 0, 2, 0, -1, 0, 0, 0),
                           (-1, 1, 2, 0, 0, 0, 0, 0), (-2, 3, 3, 0, -2, 0, 0, 0), (3, -2, -2, -1, 3, 1, 1, -1),
                           (1, -1, -1, 1, 2, 0, 0, 0), (0, 1, 0, 1, 0, 0, 0, 0), (0, 0, 1, 0, 0, 1, 0, 0),
                           (3, -2, -1, -2, 1, 3, 1, -1), (2, -1, 2, 0, -1, 0, 0, 0), (0, 0, 2, 0, 0, 0, 0, 0),
                           (-1, 2, 3, 0, -2, 0, 0, 0), (1, 0, 0, 1, 0, 0, 0, 0), (-1, 1, 2, 1, -1, -1, -1, 2),
                           (2, -1, 0, 1, 0, 0, 0, 0), (0, 0, 0, 1, 1, 0, 0, 0), (-1, 2, 1, 1, -1, 0, 0, 0),
                           (-1, 1, 2, 0, -1, 1, 0, 0), (0, 0, 2, 1, -1, -1, -1, 2), (-1, 1, 3, 0, -1, 0, 0, 0),
                           (0, 0, 2, 0, -1, 1, 0, 0), (0, 0, 3, 0, -1, 0, 0, 0), (2, -1, -1, 2, 1, -1, 0, 0),
                           (0, 0, -2, 0, 4, 0, 1, -1), (0, 0, -1, 2, 2, -1, 0, 0), (0, 0, 0, 4, 0, -2, -1, 1),
                           (-1, 2, 0, 2, 0, -1, 0, 0), (2, -1, -1, 0, 1, 0, 0, 1), (0, 0, -1, 0, 2, 0, 0, 1),
                           (0, 0, 0, 2, 0, -1, -1, 2), (-1, 2, 0, 0, 0, 0, 0, 1), (-1, 1, 1, 1, 0, 0, 0, 0),
                           (0, 0, 0, 0, 0, 0, -1, 3), (2, -1, -1, -1, 1, 2, 1, -1), (0, 0, -1, -1, 2, 2, 1, -1),
                           (0, 0, 0, 1, 0, 1, 0, 0), (-1, 2, 0, -1, 0, 2, 1, -1), (0, 0, 0, -1, 0, 2, 0, 1),
                           (0, 0, 0, -2, 0, 4, 1, -1), (-1, 0, 3, 0, 0, 0, 0, 0), (1, -1, -1, 2, 2, -1, 0, 0),
                           (-2, 2, 4, 0, -2, 0, 0, 0), (0, 1, 0, 2, 0, -1, 0, 0), (1, -1, -1, 0, 2, 0, 0, 1),
                           (0, 1, 0, 0, 0, 0, 0, 1), (0, 0, 1, 1, 0, 0, 0, 0), (1, -1, -1, -1, 2, 2, 1, -1),
                           (0, 1, 0, -1, 0, 2, 1, -1), (-1, 1, 4, 0, -2, 0, 0, 0), (1, 0, 0, 2, 0, -1, 0, 0), 
                           (1, 0, 0, 0, 0, 0, 0, 1), (1, 0, 0, -1, 0, 2, 1, -1), (0, 0, 4, 0, -2, 0, 0, 0), 
                           (2, -1, 0, 2, 0, -1, 0, 0), (0, 0, -1, 0, 3, 0, 1, -1), (0, 0, 0, 2, 1, -1, 0, 0), 
                           (-1, 2, 1, 2, -1, -1, 0, 0), (2, -1, 0, 0, 0, 0, 0, 1), (0, 0, 0, 0, 1, 0, 0, 1),
                           (-1, 2, 1, 0, -1, 0, 0, 1), (-1, 1, 2, 1, -1, 0, 0, 0), (0, 0, 1, 0, -1, 0, -1, 3), 
                           (2, -1, 0, -1, 0, 2, 1, -1), (0, 0, 0, -1, 1, 2, 1, -1), (-1, 2, 1, -1, -1, 2, 1, -1),
                           (0, 1, 1, 0, -1, 0, 0, 1), (0, 0, 2, 1, -1, 0, 0, 0), (1, -1, 0, -1, 1, 2, 1, -1),
                           (0, 0, 0, 3, 0, -2, -1, 2), (-1, 2, 0, 1, 0, -1, 0, 1), (-1, 1, 1, 2, 0, -1, 0, 0), 
                           (0, 0, 0, 1, 0, -1, -1, 3), (0, 0, -1, 0, 2, 1, 1, -1), (0, 0, 0, 2, 0, 0, 0, 0),
                           (0, 0, 0, 0, 0, 1, 0, 1), (0, 0, 0, -1, 0, 3, 1, -1), (0, 1, 0, 1, 0, -1, 0, 1),
                           (0, 0, 1, 2, 0, -1, 0, 0), (0, 0, 1, 0, 0, 0, 0, 1), (1, -1, -1, 0, 2, 1, 1, -1), 
                           (0, 0, 1, -1, 0, 2, 1, -1), (-1, 2, 1, 1, -1, -1, 0, 1), (-1, 1, 2, 2, -1, -1, 0, 0),
                           (-1, 1, 2, 0, -1, 0, 0, 1), (0, 0, 0, 0, 1, 1, 1, -1), (-1, 1, 2, -1, -1, 2, 1, -1), 
                           (0, 2, 0, 0, -1, -1, 0, 2), (-2, 1, 6, 0, -3, 0, 0, 0), (0, 1, 1, 1, -1, -1, 0, 1),
                           (0, 0, 2, 2, -1, -1, 0, 0), (0, 0, 2, 0, -1, 0, 0, 1), (-1, 1, 0, 3, 0, -1, 0, 0),
                           (1, 0, -1, -1, 1, 1, 1, 0), (1, -1, 0, 0, 1, 1, 1, -1), (0, 0, 2, -1, -1, 2, 1, -1),
                           (2, -2, -2, -2, 3, 3, 2, -2), (0, 0, 0, 3, 0, -1, 0, 0), (2, -1, -1, -1, 1, 1, 1, 0), 
                           (0, 0, 0, 1, 0, 0, 0, 1), (0, 0, 0, 0, 0, 2, 1, -1), (1, 0, 0, -1, 0, 1, 1, 0),
                           (2, -1, 0, -1, 0, 1, 1, 0), (-1, 1, 0, 4, 0, -2, 0, 0), (1, 0, -1, 0, 1, 0, 1, 0),
                           (0, 0, 0, 4, 0, -2, 0, 0), (2, -1, -1, 0, 1, 0, 1, 0), (0, 0, -1, 0, 2, 0, 1, 0), 
                           (0, 0, 0, 2, 0, -1, 0, 1), (-1, 2, 0, 0, 0, 0, 1, 0), (0, 0, 0, 0, 0, 0, 0, 2),
                           (0, 0, 0, 1, 0, 1, 1, -1), (0, 0, 0, -1, 0, 2, 1, 0), (0, 0, 0, -2, 0, 4, 2, -2),
                           (1, -1, -1, 0, 2, 0, 1, 0), (0, 1, 0, 0, 0, 0, 1, 0), (1, 0, 0, 0, 0, 0, 1, 0), 
                           (2, -1, 0, 0, 0, 0, 1, 0), (0, 0, 0, 0, 1, 0, 1, 0), (-1, 2, 1, 0, -1, 0, 1, 0),
                           (0, 0, 0, 1, 0, -1, 0, 2), (0, 0, 0, 0, 0, 1, 1, 0), (0, 0, 1, 0, 0, 0, 1, 0),
                           (-1, 1, 2, 0, -1, 0, 1, 0), (0, 0, 2, 0, -1, 0, 1, 0), (1, 0, -1, -1, 1, 1, 2, -1),
                           (0, 0, 0, 0, 0, -1, 0, 3), (2, -1, -1, -1, 1, 1, 2, -1), (0, 0, -1, -1, 2, 1, 2, -1),
                           (0, 0, 0, 1, 0, 0, 1, 0), (-1, 2, 0, -1, 0, 1, 2, -1), (0, 0, 0, -1, 0, 1, 1, 1),
                           (0, 0, 0, -2, 0, 3, 2, -1), (1, -1, -1, -1, 2, 1, 2, -1), (0, 1, 0, -1, 0, 1, 2, -1),
                           (1, 0, 0, -1, 0, 1, 2, -1), (2, -1, 0, -1, 0, 1, 2, -1), (0, 0, 0, -1, 1, 1, 2, -1),
                           (-1, 2, 1, -1, -1, 1, 2, -1), (1, 0, -1, 0, 1, 0, 2, -1), (2, -1, -1, 0, 1, 0, 2, -1),
                           (0, 0, -1, 0, 2, 0, 2, -1), (0, 0, 0, 2, 0, -1, 1, 0), (-1, 2, 0, 0, 0, 0, 2, -1),
                           (0, 0, 0, 0, 0, 0, 1, 1), (0, 0, 0, -1, 0, 2, 2, -1), (1, -1, -1, 0, 2, 0, 2, -1), 
                           (0, 1, 0, 0, 0, 0, 2, -1), (0, 0, 1, -1, 0, 1, 2, -1), (1, 0, 0, 0, 0, 0, 2, -1),
                           (2, 0, -1, -1, 0, 0, 2, 0), (2, -1, 0, 0, 0, 0, 2, -1), (0, 0, 0, 0, 1, 0, 2, -1), 
                           (-1, 2, 1, 0, -1, 0, 2, -1), (-1, 1, 2, -1, -1, 1, 2, -1), (3, -2, -2, -2, 2, 2, 3, -2), 
                           (0, 0, 2, -1, -1, 1, 2, -1), (0, 0, 0, 1, 0, -1, 1, 1), (0, 0, 0, 0, 0, 1, 2, -1), 
                           (0, 0, 1, 0, 0, 0, 2, -1), (-1, 1, 2, 0, -1, 0, 2, -1), (1, -1, -2, -1, 3, 1, 3, -2), 
                           (1, -1, -1, -2, 1, 3, 3, -2), (0, 0, 2, 0, -1, 0, 2, -1), (0, 0, 0, 1, 0, 0, 2, -1),
                           (0, 0, 0, -1, 0, 1, 2, 0), (0, 0, 0, -2, 0, 3, 3, -2), (0, 0, 0, 2, 0, -1, 2, -1),
                           (0, 0, 0, 0, 0, 0, 2, 0), (0, 0, 0, -1, 0, 2, 3, -2), (0, 0, 0, -1, 0, 1, 3, -1),
                           (4, -2, -2, -2, 1, 1, 4, -2), (0, 0, -1, 0, 1, 0, 3, -1), (0, 0, 0, 0, 0, 0, 3, -1), 
                           (0, 0, 0, -1, 0, 0, 3, 0), (0, 0, 0, -2, 0, 2, 4, -2), (0, 0, 0, -1, 0, 1, 4, -2),
                           (0, 0, -1, 0, 1, 0, 4, -2), (0, 0, 0, 0, 0, 0, 4, -2), (0, 0, 0, -2, 0, 1, 6, -3)] )

if __name__ == '__main__':
    unittest.main(verbosity = 2)
